@@ -2,11 +2,18 @@ open Str
 
 let compute_predicate () = 
   let s = Sys.ocaml_version in
-  if string_match (regexp "^3\\.\\(0[7-9]\\|[1-9]\\).*$") s 0 then
-    "3.07"
-  else if string_match (regexp "3\\.06+\\([0-9]+\\)$") s 0 then
-    let pv = matched_group 1 s in
-    let pv = int_of_string pv in
+  if string_match (regexp "^3\\.\\(0[8-9]\\|[1-9]\\).*$") s 0 then
+    "3.07_19"
+  else if string_match (regexp "^3\\.07\\+\\([0-9]+\\)\\([^0-9].*\\)?$") s 0 then
+    let pv = int_of_string (matched_group 1 s) in
+    if pv >= 19 then
+      "3.07_19"
+    else if pv >= 5 then
+      "3.07_5"
+    else
+      "3.07"
+  else if string_match (regexp "3\\.06\\+\\([0-9]+\\)$") s 0 then
+    let pv = int_of_string (matched_group 1 s) in
     if pv >= 37 then
       "3.07pre"
     else
@@ -16,7 +23,7 @@ let compute_predicate () =
   else
     failwith ("unknown or unsupported OCaml version: " ^ s)
 
-let if_regexp = regexp "^#if +\\([a-z0-9.| ]+\\) *$"
+let if_regexp = regexp "^#if +\\([a-z0-9._| ]+\\) *$"
 (* let elif_regexp = regexp "^#elif +\\([a-z0-9.| ]+\\) *$" *)
 let else_regexp = regexp "^#else *$"
 let endif_regexp = regexp "^#endif *$"
@@ -71,6 +78,8 @@ open Arg
 
 let ifname = ref None
 let pp = ref None
+let verbose = ref false
+let no_remove = ref false
 
 let arg_set r doc v = 
   match !r with
@@ -84,6 +93,8 @@ let show_pred () =
 let argspec = 
   [ "-pp", String (arg_set pp "-pp"), "\tpass output to secondary preprocessor";
     "-showpred", Unit show_pred, "\tshow defined predicate";
+    "-verbose", Set verbose, "\tverbose message";
+    "-no-remove", Set no_remove, "\tdo not remove temporary file";
   ]
 
 let anonfun = arg_set ifname "input files"
@@ -95,7 +106,8 @@ let () = Random.self_init ()
 let msg_EEXIST = Unix.error_message Unix.EEXIST
 
 let rec open_out_temp () = 
-  let fname = "camlpp" ^ (string_of_int (Random.int max_int)) in
+  let fname = "camlpp" ^ (string_of_int (Random.bits ())) in
+  if !verbose then Printf.eprintf "version_filter: opening temporary file %s\n" fname;
   try
     let fd = open_out_gen [Open_wronly; Open_creat; Open_excl] 0o666 fname in
     fd, fname
@@ -112,29 +124,47 @@ let main () =
     | Some v -> v
   in
   let pred = compute_predicate () in
+  if !verbose then Printf.eprintf "version_filter: reading %s\n" ifname;
   let ifd = open_in ifname in
   begin
     match !pp with
-      None -> process_file ifname stdout pred
+      None ->
+	if !verbose then Printf.eprintf "version_filter: output to stdout\n";
+	process_file ifname stdout pred
     | Some pp -> begin
 	let ofd, ofname = open_out_temp () in
 	try
 	  begin
 	    try
+	      if !verbose then Printf.eprintf "version_filter: output to %s\n" ofname;
 	      process_file ifname ofd pred;
+	      if !verbose then Printf.eprintf "version_filter: closing %s\n" ofname;
 	      close_out ofd
 	    with 
 	      e -> close_out ofd; raise e
 	  end;
-	  let rstatus = Sys.command (pp ^ " " ^ Filename.quote ofname) in
+	  let cmdline = (pp ^ " " ^ Filename.quote ofname) in
+	  if !verbose then Printf.eprintf "version_filter: invoking %s\n" cmdline;
+	  let rstatus = Sys.command cmdline in
+	  if !verbose then Printf.eprintf "version_filter: command %s returns %d\n" cmdline rstatus;
 	  if rstatus <> 0 then
 	    failwith "secondary preprocessor failed";
-	  Sys.remove ofname
+	  if not !no_remove then begin
+	    if !verbose then Printf.eprintf "version_filter: removing %s\n" ofname;
+	    Sys.remove ofname;
+	    if !verbose then Printf.eprintf "version_filter: removing %s done\n" ofname
+	  end
 	with
 	  e ->
-	    Sys.remove ofname; raise e
+	    if not !no_remove then begin
+	      if !verbose then Printf.eprintf "version_filter: removing %s (err)\n" ofname;
+	      Sys.remove ofname; 
+	      if !verbose then Printf.eprintf "version_filter: removing %s done\n" ofname
+	    end;
+	    raise e
     end
   end
 
 let () = main ()
 
+let () = if !verbose then Printf.eprintf "version_filter: exiting\n";
