@@ -1,3 +1,6 @@
+(*pp camlp4o -I . *)
+#load "pa_once.cma";;
+
 module A = 
   struct
     let f () = once 1
